/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.LoginPage;
import org.alfresco.po.share.SharePage;

/**
 * Process an Alfresco share login by mimicking real user login.
 * 
 * <h1>Input</h1>
 * 
 * {@link ShareEvent} The information includes {@link WebDrone}, username and the Url (e.g https://benchmy.alfresco.me) 
 * 
 * <h1>Actions</h1>
 * 
 * Uses the Web Drone to navigate to an Alfresco url, Once the login page is displayed
 * it then populates the login form with the username and password that is fetched from
 * the MongoDB using the username.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_LOGIN_COMPLETE}:  A populated {@link ShareEventData#getSharePage() Share page}<br/>
 * 
 * @author Michael Suzuki
 * @since 1.0
 */
public class LoginEventProcess extends AbstractShareEventProcessor
{
    private final static String EVENT_NAME_LOGIN_COMPLETE = "share.login.complete";
    private final static String ERROR_MSG_SHARE_EVENT_DATA = "LoginEventProcess failed, ShareEventData is required";
    private final static String ERROR_MSG_USERNAME = "LoginEventProcess failed, username can not be null";

    public LoginEventProcess(UserDataService userDataService)
    {
        super(userDataService, EVENT_NAME_LOGIN_COMPLETE);
    }
    
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        suspendTimer();
        
        EventResult result = null;
        if(eventData == null)
        {
            return new EventResult(ERROR_MSG_SHARE_EVENT_DATA, false);
        }
        
        String username = eventData.getUsername();
        if (username == null || username.trim().isEmpty())
        {
            return new EventResult(ERROR_MSG_USERNAME, false);
        }
        
        UserData user = userDataService.findUserByUsername(username);
        if(user == null)
        {
            String msg = String.format("LoginEventProcess failed, unable to log user: %s" +
                                       "  user not found.",username);
            return new EventResult(msg, false);
        }
        
        String password = user.getPassword();
        LoginPage loginPage = (LoginPage) eventData.getSharePage();
        loginPage.loginAs(username, password);
        
        resumeTimer();
        SharePage page = (SharePage) loginPage.getDrone().getCurrentPage().render();
        stopTimer();
//        if (page.isLoggedIn())  
//        {    
            eventData.setSharePage(page);
            Event nextEvent = new Event(
                    eventNameActionComplete,
                    eventData);
            result = new EventResult(
                    String.format(
                            "%s has logged in successfully to %s (JESSIONID: %s)",
                            username, eventData.getUrl(), page.getDrone().getSessionId()),
                    nextEvent);
//        } 
//        else
//        {
//            String msg = String.format("Failed to login user:%s against url: %s",
//                                       username,
//                                       eventData.getUrl());
//            result = new EventResult(msg, false);
//        }
        return result;
    }

}
