/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.entity;

import org.alfresco.bm.event.Event;
import org.alfresco.po.share.SharePage;
import org.alfresco.webdrone.WebDrone;

/**
 * Wrapper for {@link Event} that holds the user session information which are:
 * <ul>
 *     <li>URL</li>
 *     <li>Username</li>
 *     <li>{@link SharePage}</li>
 *     <li>Site name</li>
 * </ul>
 * 
 * This is the minimum required data to start the user journey.
 * 
 * @author Michael Suzuki
 * @since 1.0
 */
public class ShareEventData
{
    /*
     * TODO: Everything in here should be final.
     *       Copy constructors will make it easy to produce instances with
     *       new values.  Derived types should be used for the different
     *       flavours of data rather than having this type overloaded with
     *       nuances of specific scenarios. See ShareStartEventData and
     *       SharePagingEventData.
     */
    
    private final String username;
    private final String url;
    private SharePage sharePage;
    // TODO: The values below should be final and in derived, event-specific types
    private String siteName;
    private String search;
    private String filename;

    /**
     * Copy constructor.
     * <p/>
     * Copies the critical values from the provided source
     * 
     * @param copyFrom          the instance to copy basic values from
     */
    public ShareEventData(ShareEventData copyFrom)
    {
        this.username = copyFrom.username;
        this.url = copyFrom.url;
        this.sharePage = copyFrom.sharePage;
        this.filename = copyFrom.filename;
    }
    
    /**
     * Constructor.
     * @param username String username
     * @param url String alfresco url
     */
    public ShareEventData(final String username, final String url)
    {
        this.username = username;
        this.url = url;
    }
    
    @Override
    public String toString()
    {
        String sharePageStr;
        try
        {
            sharePageStr = "" + sharePage;
        }
        catch (Exception e)
        {
            // It happens
            sharePageStr = "[SharePage.toString() failed: " + e.getMessage() + "]";
        }
        StringBuilder builder = new StringBuilder();
        builder.append("ShareEventData [username=");
        builder.append(username);
        builder.append(", url=");
        builder.append(url);
        builder.append(", sharePage=");
        builder.append(sharePageStr);
        builder.append(", siteName=");
        builder.append(siteName);
        builder.append(", search=");
        builder.append(search);
        builder.append(", filename=");
        builder.append(filename);
        builder.append("]");
        return builder.toString();
    }

    /**
     * Static class that indicates type
     * of find site to use which allows empty results
     * for site search.
     * 
     * @author Michael Suzuki
     *
     */
    public static class FindSite extends ShareEventData
    {
        public FindSite(ShareEventData data)
        {
            super(data.getUsername(), data.getUrl());
            this.setSharePage(data.getSharePage());
            this.setSiteName(data.getSiteName());
        }
    }
    
    /**
     * Static class that indicates type
     * of find site that requires a positive result for site search.
     * 
     * @author Michael Suzuki
     *
     */
    public static class MustFindSite extends ShareEventData
    {
        public MustFindSite(ShareEventData data)
        {
            super(data.getUsername(), data.getUrl());
            this.setSharePage(data.getSharePage());
            this.setSiteName(data.getSiteName());
        }
    }

    /**
     * Gets the {@link WebDrone} out of the {@link SharePage}
     * 
     * @return {@link WebDrone} from the share page.
     */
    public WebDrone getWebDrone()
    {
        WebDrone drone = null;
        if (sharePage != null)
        {
            drone = sharePage.getDrone();
        }
        return drone;
    }

    public String getUsername()
    {
        return username;
    }

    public String getUrl()
    {
        return url;
    }

    public SharePage getSharePage()
    {
        return sharePage;
    }

    public void setSharePage(SharePage sharePage)
    {
        this.sharePage = sharePage;
    }

    public String getSiteName()
    {
        return siteName;
    }

    public void setSiteName(String siteName)
    {
        this.siteName = siteName;
    }

    public String getSearch()
    {
        return search;
    }

    public void setSearch(String search)
    {
        this.search = search;
    }

    public String getFilename()
    {
        return filename;
    }

    public void setFilename(String filename)
    {
        this.filename = filename;
    }
}
