/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import java.io.File;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.file.FileData;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareDownloadEventData;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.site.document.DocumentDetailsPage;
import org.alfresco.webdrone.HtmlPage;

/**
 * Click and download the file
 *  
 * <h1>Input</h1> 
 * 
 * {@link DocumentDetailsPage} showing the file details
 * 
 * <h1>Actions</h1>
 * 
 * Downloads the file to local disk
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_DOWNLOAD_DOCUMENT_COMPLETE}: {@link DocumentDetailsPage} in {@link ShareEventData}<br/>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class FileDownloadEventProcess extends AbstractShareEventProcessor
{
    public static final String EVENT_NAME_DOWNLOAD_DOCUMENT_COMPLETE = "share.docDetails.downloadDocument.complete";
    public static final boolean DEFAULT_DOWNLOAD_VIA_CLICK = false;
    
    private File downloadDir = null;
    private boolean downloadViaClick = DEFAULT_DOWNLOAD_VIA_CLICK;
    
    /**
     * @param userDataService           service to provide user data
     * @param downloadDirectory         the directory where downloaded files are stored.
     *                                  Use empty string (or <tt>null</tt>) to indicate that the location
     *                                  is unknown.
     */
    public FileDownloadEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_DOWNLOAD_DOCUMENT_COMPLETE);
    }

    /**
     * Override the {@link #DEFAULT_DOWNLOAD_VIA_CLICK default} method of executing the file
     * download (default:<b>false</b>).
     * 
     * @param downloadViaClick          <tt>true</tt> to have the browser click the link and
     *                                  download the file to the browser's download directory;
     *                                  the is less reliable than <tt>true</tt>, which uses a
     *                                  HTTP client to download the file directly.
     */
    public void setDownloadViaClick(boolean downloadViaClick)
    {
        this.downloadViaClick = downloadViaClick;
    }

    /**
     * Set the location where the browser will be putting files.  If this is not known, do not set
     * anything.
     * 
     * @param downloadDir               a path to a folder that must exist (<tt>null</tt> allowed)
     */
    public void setDownloadDir(String downloadDir)
    {
        if (downloadDir == null || downloadDir.length() == 0)
        {
            // Leave it null if we are not using it
            this.downloadDir = null;
        }
        else
        {
            this.downloadDir = new File(downloadDir);
            if (!this.downloadDir.exists())
            {
                // Attempt to make it
                this.downloadDir.mkdirs();
                if (!this.downloadDir.exists())
                {
                    throw new IllegalArgumentException("Download directory not found and could not be created: " + downloadDir);
                }
            }
            else if (this.downloadDir.isFile())
            {
                throw new IllegalArgumentException("Not a directory: " + downloadDir);
            }
        }
    }

    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        DocumentDetailsPage docDetailsPage = (DocumentDetailsPage) eventData.getSharePage();
        if (!(eventData instanceof ShareDownloadEventData))
        {
            throw new RuntimeException("Share event data must be " + ShareDownloadEventData.class.getName());
        }
        ShareDownloadEventData downloadData = (ShareDownloadEventData) eventData;
        // Get the title of the document
        String title = docDetailsPage.getDocumentTitle();

        // Now download the file
        HtmlPage page = null;
        if (downloadViaClick)
        {
            // Just get the browser to download it to wherever
            page = docDetailsPage.selectDownload(null);
        }
        else
        {
            String extension = FileData.getExtension(title);
            String baseTitle = title;
            // Get the base name of the file
            int dotIndex = title.lastIndexOf(".");
            if (dotIndex >= 0)
            {
                baseTitle = title.substring(0, dotIndex);
            }
            // Create a temporary file, if necessary
            File givenTempFile = downloadData.getFile();
            File tempFile = givenTempFile;
            if (givenTempFile == null)
            {
                // We were not told where to put the file, so just choose a place
                tempFile = File.createTempFile(baseTitle, "." + extension, downloadDir);
                tempFile.deleteOnExit();
            }
            else
            {
                tempFile = givenTempFile;
            }
            page = docDetailsPage.selectDownload(tempFile);
            // Double-check the file
            if (!tempFile.exists())
            {
                EventResult result = new EventResult("File was not downloaded: " + title, false);
                return result;
            }
            else
            {
                // Double check the size
                // TODO: Get the document size
                if (givenTempFile == null)
                {
                    // Delete it
                    try { tempFile.delete(); } catch (Throwable e) {}
                }
                else
                {
                    // We leave it for the caller
                }
            }
        }
        
        // Need to render again (should be quick)
        docDetailsPage = page.render();
        eventData.setSharePage(docDetailsPage);
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        EventResult result = new EventResult(
                "Downloaded file " + title,
                nextEvent);
        return result;
    }
}
