/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.ShareEventProcessException;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareSelectDocLibEntryEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.site.document.DocumentLibraryPage;
import org.alfresco.webdrone.HtmlPage;

/**
 * Select folder event process: mimics the action of selecting a folder
 * on the document library page.
 *  
 * <h1>Input</h1>
 *  
 * {@link ShareEventData} containing {@link DocumentLibraryPage} and with the
 * {@link ShareEventData#getFilename() filename} populated.
 * 
 * <h1>Actions</h1>
 * 
 * Clicks the file or folder.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_SELECT_FOLDER_COMPLETE}: {@link DocumentLibraryPage} in {@link ShareEventData}<br/>
 * 
 * @author Michael Suzuki
 * @author Derek Hulley
 * @since 1.0
 */
public class SelectDocLibFolderEventProcess extends AbstractShareEventProcessor
{
    public static final String EVENT_NAME_SELECT_FOLDER_COMPLETE = "share.doclib.selectFolder.complete";

    public SelectDocLibFolderEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_SELECT_FOLDER_COMPLETE);
    }
    
    /**
     * Process which selects the file link.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        // Pause timer: We only want the render call to be timed
        suspendTimer();
        
        if (!(eventData instanceof ShareSelectDocLibEntryEventData))
        {
            throw new ShareEventProcessException("Expect a 'ShareSelectDocLibEntryEventData' for selecting entries.");
        }
        ShareSelectDocLibEntryEventData entryEventData = (ShareSelectDocLibEntryEventData) eventData;
        String folderName = entryEventData.getEntry();

        DocumentLibraryPage docPage = (DocumentLibraryPage) eventData.getSharePage();
        HtmlPage page = docPage.selectFolder(folderName);
        // Resume timer: We only want the render call timed
        resumeTimer();
        DocumentLibraryPage doclibPage = page.render();
        
        eventData.setSharePage(doclibPage);
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        EventResult result = new EventResult(
                "Folder selected: " + eventData,
                nextEvent);
        return result;
    }
}
