/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import java.io.File;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.file.FileData;
import org.alfresco.bm.file.TestFileService;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.site.UpdateFilePage;
import org.alfresco.po.share.site.UploadFilePage;
import org.alfresco.po.share.site.document.DocumentDetailsPage;

/**
 * Selects the <i>Upload New Verson</i> link in the document details page.
 *  
 * <h1>Input</h1> 
 * 
 * {@link DocumentDetailsPage} The page object showing the document details.
 * 
 * <h1>Actions</h1>
 * 
 * Selects the <i>Upload New Version</i> link.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_SELECT_UPLOAD_NEW_VERSION_COMPLETE}: {@link UploadFilePage} in {@link ShareEventData}<br/>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class SelectUploadNewVersionEventProcess extends AbstractShareEventProcessor
{
    public static final String EVENT_NAME_SELECT_UPLOAD_NEW_VERSION_COMPLETE = "share.docDetails.selectUploadNewVersion.complete";
    
    private final TestFileService testFileService;
    
    /**
     * @param userDataService           service to provide user data
     * @param testFileService           service to provide test files
     */
    public SelectUploadNewVersionEventProcess(UserDataService userDataService, TestFileService testFileService)
    {
        super(userDataService,EVENT_NAME_SELECT_UPLOAD_NEW_VERSION_COMPLETE);
        this.testFileService = testFileService;
    }
    
    /**
     * Select {@link DocumentDetailsPage#uploadNewVersion()}
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        DocumentDetailsPage docLibPage = (DocumentDetailsPage) eventData.getSharePage();

        File file = null;
        String filename = eventData.getFilename();
        if (filename == null)
        {
            filename = docLibPage.getDocumentTitle();
            String extension = FileData.getExtension(filename);
            file = testFileService.getFile(extension);
            if (file == null)
            {
                throw new RuntimeException("Unable to find a test file with the same extension: " + filename);
            }
            filename = file.getCanonicalPath();
        }
        else
        {
            file = new File(filename);
        }

        UpdateFilePage updateFilePage = docLibPage.selectUploadNewVersion().render();
        eventData.setSharePage(updateFilePage);
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        EventResult result = new EventResult(
                "Clicked upload new version for " + filename + " on page " + docLibPage,
                nextEvent);
        return result;
    }
}
