/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.site.CreateSitePage;
import org.alfresco.po.share.site.SiteDashboardPage;

/**
 * An Alfresco share site create process, mimics the action of completing the site
 * creation form and submitting the form.
 *  
 * <h1>Input</h1>
 * 
 * The following wrapped in {@link ShareEventData}:<br/>
 * {@link CreateSitePage} The page object representing the create site form to complete.<br/>
 * {@link ShareEventData#getSiteName() A site name}<br/>
 * 
 * <h1>Actions</h1>
 * 
 * Populates the create site form with siteName and submits it to create site.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_SITE_CREATED}: {@link SiteDashboardPage} in a {@link ShareEventData}<br/>
 * 
 * @author Michael Suzuki
 * 
 * @since 1.0
 */
public class SiteCreateEventProcess extends AbstractShareEventProcessor
{
    private static final String ERROR_MISSING_SITENAME = "Create site action failed, siteName is required";
    private static final String EVENT_NAME_SITE_CREATED = "share.site.created";

    public SiteCreateEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_SITE_CREATED);
    }
    
    /**
     * Process which fills and submits the create site form.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        String siteName = eventData.getSiteName();
        if (siteName == null)
        {
            return new EventResult(ERROR_MISSING_SITENAME, false);
        }
        
        CreateSitePage createSitePage = (CreateSitePage) eventData.getSharePage();
        SiteDashboardPage site = createSitePage.createNewSite(siteName).render();
        eventData.setSharePage(site);
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        EventResult result = new EventResult(
                "Created site: " + siteName,
                nextEvent);
        return result;
    }
}
