/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import java.lang.reflect.Method;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.ShareEventProcessException;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.Navigation;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.site.SiteNavigation;
import org.alfresco.po.share.site.SitePage;

/**
 * An Alfresco share site sub navigation process that mimics the action of
 * selecting a button on Alfresco's site sub navigation bar.
 *  
 * <h1>Input</h1>
 *  
 * A {@link SitePage} that is displayed to the user.
 * 
 * <h1>Actions</h1>
 * 
 * Uses the {@link Navigation} on the share page to navigate from the current rendered
 * page to the destination page using the main navigation bar.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_NAVIGATION_COMPLETED}: Valid page using {@link ShareEventData}<br/>
 * 
 * @author Michael Suzuki
 * 
 * @since 1.0
 */
public class SiteNavigationEventProcess extends AbstractShareEventProcessor
{
    static final String SELECT_DOCUMENT_LIB = "selectSiteDocumentLibrary";
    static final String SELECT_SITE_DASHBOARD = "selectSiteDashBoard";
    
    private final String navigateTo;

    /**
     * Constructor.
     * 
     * @param navigateTo    method name to select on the {@link Navigation}
     */
    public SiteNavigationEventProcess(
            UserDataService userDataService,
            String siteNavigationActionName,
            String navigateTo)
    {
        super(userDataService, siteNavigationActionName);
        this.navigateTo = navigateTo;
    }
    
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        // Get the button from navigation to click
        if (navigateTo == null || navigateTo.isEmpty())
        { 
            return new EventResult(
                "Navigation eventNameActionComplete failed, navigate to parameter not found.",
                false);
        }
        return navigateTo(eventData);
    }


    /**
     * Site main navigation, the event of selecting on the main navigation.
     * 
     * @param {@link Event} event data
     * @return {@link EventResult} of the eventNameActionComplete
     */
    private EventResult navigateTo(final ShareEventData eventData) 
    {
        EventResult eventResult = null;
        SitePage page = eventData.getSharePage().render();
        try
        {
            SiteNavigation nav = page.getSiteNav();
            Method method = nav.getClass().getMethod(navigateTo, new Class[] {});
            SharePage response = (SharePage) method.invoke(nav, new Object[] {});
            response = response.render();
            eventData.setSharePage(response);
            Event nextEvent = new Event(
                    eventNameActionComplete,
                    eventData);
            eventResult = new EventResult(
                    "Navigated to '" + navigateTo + "' from page " + page,
                    nextEvent);
        }
        catch (NoSuchMethodException e)
        {
            eventResult = new EventResult(
                    "Navigation method does not exist: " + navigateTo,
                    false);
        }
        catch (Throwable throwable)
        {
            throw new ShareEventProcessException(page.getDrone(), throwable);
        }
        return eventResult;
    }
}
