/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.util.Collections;
import java.util.List;
import java.util.UUID;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.po.share.DashBoardPage;
import org.alfresco.po.share.ShareLink;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.dashlet.Dashlet;
import org.alfresco.po.share.dashlet.MySitesDashlet;
import org.alfresco.po.share.site.CreateSitePage;

/**
 * Browses to the user dashboard.
 * <p/>
 * The following page is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. User dashboard:</b> Checks the  {@link #setMaxSitesPerUser(int) site count}</li>
 *   <li><b>*  2. Create site page:</b> Choose a randomly-named site</li>
 *   <li><b>   3. Site dashboard:</b></li>
 * </ul>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareCreateSiteScenario extends AbstractShareScenario
{
    public static final String EVENT_NAME_NAVIGATE_TO_CREATE_SITE = "share.nav.createSite";
    public static final String EVENT_NAME_CREATE_SITE = "share.site.create";
    public static final String DEFAULT_SITE_NAME_PREFIX = "";
    public static final int DEFAULT_MAX_SITES_PER_USER = 20;

    private int maxSitesPerUser = DEFAULT_MAX_SITES_PER_USER;
    private String eventNameNavigateToCreateSite = EVENT_NAME_NAVIGATE_TO_CREATE_SITE;
    private String eventNameCreateSite = EVENT_NAME_CREATE_SITE;
    private String siteNamePrefix = DEFAULT_SITE_NAME_PREFIX;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     */
    public ShareCreateSiteScenario(ShareScenarioRegistry registry, double weight)
    {
        super(registry, weight);
    }

    /**
     * Override the {@link #DEFAULT_MAX_SITES_PER_USER default} maximum number of sites per user.
     * When this number of sites is reached (visible in the 'My Sites' dashlet) then this
     * scenario will cease to trigger.
     */
    public void setMaxSitesPerUser(int maxSitesPerUser)
    {
        this.maxSitesPerUser = maxSitesPerUser;
    }

    /**
     * Override the {@link #EVENT_NAME_NAVIGATE_TO_CREATE_SITE default} name for events
     * to navigate to the create site page
     */
    public void setEventNameNavigateToCreateSite(String eventNameNavigateToCreateSite)
    {
        this.eventNameNavigateToCreateSite = eventNameNavigateToCreateSite;
    }

    /**
     * Override the {@link #EVENT_NAME_CREATE_SITE default} name for events to create a site
     */
    public void setEventNameCreateSite(String eventNameCreateSite)
    {
        this.eventNameCreateSite = eventNameCreateSite;
    }

    /**
     * Override the {@link #DEFAULT_SITE_NAME_PREFIX default} site name prefix
     */
    public void setSiteNamePrefix(String siteNamePrefix)
    {
        this.siteNamePrefix = siteNamePrefix;
    }

    /**
     * @see ShareCreateSiteScenario see class comments
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        SharePage sharePage = eventData.getSharePage();
        if (sharePage instanceof DashBoardPage)
        {
            // This is the trigger for the scenario.  Find out how many sites the user has.
            DashBoardPage dashboardPage = (DashBoardPage) sharePage;
            Dashlet dashlet = dashboardPage.getDashlet("my-sites");
            if (dashlet == null)
            {
                // The user has no such dashlet
                return Collections.emptyList();
            }
            MySitesDashlet mySitesDashlet = dashlet.render();       // ClassCastException is OK
            List<ShareLink> siteLinks = mySitesDashlet.getSites();
            if (siteLinks.size() >= maxSitesPerUser)
            {
                // There are enough sites
                return Collections.emptyList();
            }
            // We need to browse to the create site link
            Event nextEvent = new Event(eventNameNavigateToCreateSite, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else if (sharePage instanceof CreateSitePage)
        {
            // We're looking at the form.  Choose a random site name.
            String site = siteNamePrefix + UUID.randomUUID().toString();
            eventData.setSiteName(site);
            Event nextEvent = new Event(eventNameCreateSite, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else
        {
            // We terminate because this is where the scenario ends
            return Collections.emptyList();
        }
    }
}
