/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.util.HashMap;
import java.util.Map;
import java.util.NavigableMap;
import java.util.Random;
import java.util.TreeMap;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;

import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.EventProcessor;
import org.alfresco.bm.event.selector.RandomWeightedEventSelector;

/**
 * A registry of {@link EventProcessor event processors} that allow for simple
 * {@link AbstractEventProcessor#register() registration}
 * 
 * @see RandomWeightedEventSelector RandomWeightedEventSelector for details on weighting implementation.
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareScenarioRegistry
{
    private final Map<String, ShareScenario> scenarios;
    private final ReadLock readLock;
    private final WriteLock writeLock;
    private final NavigableMap<Double, String> scenarioWeights = new TreeMap<Double, String>();
    private double totalWeights = 0.0;
    private final Random random = new Random(System.currentTimeMillis());;
    
    public ShareScenarioRegistry()
    {
        this.scenarios = new HashMap<String, ShareScenario>(97);
        ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
        readLock = lock.readLock();
        writeLock = lock.writeLock();
    }
    
    /**
     * Register a share scenario by name and with the given weighting
     * 
     * @param scenarioName      the name of the scenario
     * @param scenario          the scenario implementation
     */
    public void register(String scenarioName, ShareScenario scenario, double weight)
    {
        writeLock.lock();
        try
        {
            scenarios.put(scenarioName, scenario);
            // Zero-weighted options would crowd the end of the range
            if (weight > 0.0)
            {
                totalWeights += weight;
                scenarioWeights.put(totalWeights, scenarioName);
            }
        }
        finally
        {
            writeLock.unlock();
        }
    }
    
    /**
     * Get the processor that must handle events with the given name
     * 
     * @param scenarioName      the name of the event scenario
     * @return                  a scenario implementation or <tt>null</tt> if it is unmapped
     */
    public ShareScenario getScenario(String scenarioName)
    {
        readLock.lock();
        try
        {
            return scenarios.get(scenarioName);
        }
        finally
        {
            readLock.unlock();
        }
    }
    
    /**
     * Choose a random scenario based on the weightings provided on registration
     * 
     * @return                  the name of a scenario
     */
    public String chooseRandomScenario()
    {
        if (scenarioWeights.size() == 0)
        {
            throw new RuntimeException("Nothing to choose from.");
        }
        double value = random.nextDouble() * totalWeights;
        return scenarioWeights.ceilingEntry(value).getValue();
    }
}
