/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventProcessor;
import org.alfresco.bm.event.EventProcessorRegistry;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.session.SessionService;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.random.NormalDistributionHelper;
import org.joda.time.Period;

import com.mongodb.DBObject;

/**
 * A processor that chooses the next Share user scenario based on the current session's context.
 * <p/>
 * This class makes use of a {@link EventProcessorRegistry registry} of {@link EventProcessor}
 * instances that effectively choose the next event(s) to execute.<br/>
 * Events are extracted from the normal results and modified to ensure a minimum delay in the processing.
 * <p/>
 * A user's session is always respected i.e. the user scenario will be called to determine the
 * next user action.  Where there is no active user scenario, the current session time will be
 * examined and the user either logout or a new scenario will be chosen (according to weightings).
 * Scenarios will start on the user dashboard page.
 *  
 * <h1>Input</h1>
 * 
 * {@link ShareEventData} with a valid Share page. 
 * 
 * <h1>Actions</h1>
 * 
 * No actions are performed.  This processor is merely a route selector.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_LOGOUT}: A share page is carried forward to log out
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareScenarioSelector extends AbstractShareEventProcessor
{
    public static final String EVENT_NAME_LOGOUT = "share.logout";
    public static final String DEFAULT_FALLBACK_EVENT_NAME = EVENT_NAME_LOGOUT;
    
    public static final long DEFAULT_MIN_SESSION_TIME = 120000L;
    public static final long DEFAULT_MAX_SESSION_TIME = 240000L;
    
    public static final long DEFAULT_MIN_EVENT_DELAY = 5000L;
    public static final long DEFAULT_MAX_EVENT_DELAY = 15000L;
    
    public static final int DEFAULT_SESSION_ATTEMPTS = 20;
    
    /** Bean name is used for reporting purposes */
    private String beanName;
    private final SessionService sessionService;
    private final ShareScenarioRegistry scenarioRegistry;
    private final NormalDistributionHelper normalDistribution;
    private long minSessionTime = DEFAULT_MIN_SESSION_TIME;
    private long maxSessionTime = DEFAULT_MAX_SESSION_TIME;
    private long minEventDelay = DEFAULT_MIN_EVENT_DELAY;
    private long maxEventDelay = DEFAULT_MAX_EVENT_DELAY;
    private int sessionAttempts = DEFAULT_SESSION_ATTEMPTS;
    
    private String fallbackEventName = DEFAULT_FALLBACK_EVENT_NAME;
    
    /**
     * @param userDataService           required in case user needs to log in again
     * @param sessionService            service to access session-specific data
     * @param scenarioRegistry          a registry of user scenario processors.
     */
    public ShareScenarioSelector(
            UserDataService userDataService,
            SessionService sessionService,
            ShareScenarioRegistry scenarioRegistry)
    {
        super(userDataService, null);
        this.sessionService = sessionService;
        this.scenarioRegistry = scenarioRegistry;
        this.normalDistribution = new NormalDistributionHelper();
    }

    /**
     * Overridden to record the bean name for reporting.  The base class implementation
     * is called.
     */
    @Override
    public void setBeanName(String beanName)
    {
        this.beanName = beanName;
        super.setBeanName(beanName);
    }

    /**
     * Override the {@link #DEFAULT_MIN_SESSION_TIME default} minimum user session length
     */
    public void setMinSessionTime(long minSessionTime)
    {
        this.minSessionTime = minSessionTime;
    }
    /**
     * Override the {@link #DEFAULT_MAX_SESSION_TIME default} maximum user session length
     */
    public void setMaxSessionTime(long maxSessionTime)
    {
        this.maxSessionTime = maxSessionTime;
    }

    /**
     * Override the {@link #DEFAULT_MIN_EVENT_DELAY default} minimum user wait
     */
    public void setMinEventDelay(long minEventDelay)
    {
        this.minEventDelay = minEventDelay;
    }
    /**
     * Override the {@link #DEFAULT_MAX_EVENT_DELAY default} maximum user wait
     */
    public void setMaxEventDelay(long maxEventDelay)
    {
        this.maxEventDelay = maxEventDelay;
    }

    /**
     * Override the {@link #DEFAULT_SESSION_ATTEMPTS default} number of times to try to
     * get a session that can continue from the current context.
     */
    public void setSessionAttempts(int sessionAttempts)
    {
        this.sessionAttempts = sessionAttempts;
    }

    /**
     * Override the {@link #DEFAULT_FALLBACK_EVENT_NAME default} event name to be called
     * when no scenario could be found.
     */
    public void setFallbackEventName(String fallbackEventName)
    {
        this.fallbackEventName = fallbackEventName;
    }

    /**
     * Helper method to get soak data with a target session time following a normal distribution.
     */
    private ShareSoakData getSoakData(String sessionId)
    {
        ShareSoakData soakData = null;
        DBObject soakDataDBObj = sessionService.getSessionData(sessionId);
        if (soakDataDBObj == null)
        {
            // Choose some target session length for this session
            long sessionTime = normalDistribution.getValue(minSessionTime, maxSessionTime);
            // Construct the return data
            soakData = new ShareSoakData();
            soakData.setTargetSessionDuration(sessionTime);
            // Add to the data
            // Associate the data with the session
            soakDataDBObj = ShareSoakData.toDBObject(soakData);
            sessionService.setSessionData(sessionId, soakDataDBObj);
        }
        else
        {
            soakData = ShareSoakData.fromDBObject(soakDataDBObj);
        }
        return soakData;
    }
    
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        String sessionId = event.getSessionId();
        if (sessionId == null)
        {
            throw new IllegalStateException("All Share events should be executed with an active session: " + this);
        }
        
        // Get data governing the soak test
        ShareSoakData soakData = getSoakData(sessionId);
        String scenarioName = soakData.getScenarioName();
        
        // Are we running an active scenario?
        ShareScenario scenario = null;
        if (scenarioName != null)
        {
            // Push event handling off to the scenario
            scenario = scenarioRegistry.getScenario(scenarioName);
            if (scenario == null)
            {
                scenarioName = null;
                scenario = null;
            }
        }
        
        // Some commonly-required timings
        long now = System.currentTimeMillis();
        long nextEventDelay = normalDistribution.getValue(minEventDelay, maxEventDelay);
        long nextEventTime = now + nextEventDelay;
        long sessionStart = sessionService.getSessionStartTime(sessionId);
        long sessionTargetEndTime = sessionStart + soakData.getTargetSessionDuration();

        // Try to find the next events to use
        Set<String> attemptedScenarioNames = new HashSet<String>(29);
        for (int i = 0; i < sessionAttempts; i++)
        {
            // Any existing scnerio always gets a free shot at the next events
            if (scenario == null)
            {
                // There is no currently-chosen scenario
                // Clear out any residual event data
                eventData = new ShareEventData(eventData);
                // Check session time for termination
                if (now > sessionTargetEndTime)
                {
                    // We have run out of time to choose a new scenario
                    break;
                }
                scenarioName = scenarioRegistry.chooseRandomScenario();
                scenario = scenarioRegistry.getScenario(scenarioName);
                // Don't attempt the same scenario twice
                if (!attemptedScenarioNames.add(scenarioName))
                {
                    // We've already tried it
                    scenarioName = null;
                    scenario = null;
                    continue;
                }
            }
            // We've NOT tried it, yet
            // Find out which event(s) to trigger next
            List<Event> nextEvents = scenario.getNextEvents(event, eventData, soakData, nextEventTime);
            if (nextEvents.size() == 0)
            {
                // The scenario chose nothing (it has finished).  Try another.
                scenarioName = null;
                scenario = null;
                // We are going to choose a new scenario
                continue;
            }
            else
            {
                // Modify the session data with the new choice
                soakData.setScenarioName(scenarioName);
                DBObject soakDataDBObject = ShareSoakData.toDBObject(soakData);
                sessionService.setSessionData(sessionId, soakDataDBObject);
                // We have a valid scenario
                String msg = String.format(
                        "[Session %s: %s] '%s' used '%s' to generate %d events: %s",
                        sessionId,
                        new Period(now - sessionStart).toString(),
                        beanName,
                        scenarioName,
                        nextEvents.size(),
                        nextEvents);
                EventResult result = new EventResult(msg, nextEvents);
                return result;
            }
        }
        
        // We didn't manage to find anything to do
        soakData.setScenarioName(null);
        DBObject soakDataDBObject = ShareSoakData.toDBObject(soakData);
        sessionService.setSessionData(sessionId, soakDataDBObject);
        // Session terminates with a logout
        Event nextEvent = new Event(fallbackEventName, nextEventTime, eventData);
        String msg = String.format(
                "[Session %s: %s] '%s' chose no scenario.  Routing to %s.",
                sessionId,
                new Period(now - sessionStart).toString(),
                beanName,
                fallbackEventName);
        EventResult result = new EventResult(msg, nextEvent);
        return result;
    }
}
