/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.util.Collections;
import java.util.List;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareSelectDocLibEntryEventData;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.site.document.DocumentDetailsPage;
import org.alfresco.po.share.site.document.DocumentLibraryPage;
import org.alfresco.po.share.site.document.FileDirectoryInfo;

/**
 * Selects a random entry in a site document library.
 * <p/>
 * This class uses the {@link ShareBrowseToSiteDocLibScenario parent} scenario
 * until the site document library has been reached.
 * <p/>
 * The following page is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. Site doclib (with entries):</b> Chooses a random file or folder.  Can return to this page.</li>
 *   <li><b>   2.1. Site doclib:</b> END</li>
 *   <li><b>   2.2. Document details:</b> END</li>
 * </ul>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareSelectDocLibEntryScenario extends ShareBrowseToSiteDocLibScenario
{
    public static final int DEFAULT_MAX_BROWSE_DEPTH = 7;
    
    public static final String EVENT_NAME_DOCLIB_SELECT_FILE = "share.doclib.selectFile";
    public static final String EVENT_NAME_DOCLIB_SELECT_FOLDER = "share.doclib.selectFolder";

    private int maxBrowseDepth = DEFAULT_MAX_BROWSE_DEPTH;
    private String eventNameDocLibSelectFile = EVENT_NAME_DOCLIB_SELECT_FILE;
    private String eventNameDocLibSelectFolder = EVENT_NAME_DOCLIB_SELECT_FOLDER;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     */
    public ShareSelectDocLibEntryScenario(ShareScenarioRegistry registry, double weight)
    {
        super(registry, weight);
    }

    /**
     * Override the {@link #DEFAULT_MAX_BROWSE_DEPTH} default max browse depth.
     */
    public void setMaxBrowseDepth(int maxBrowseDepth)
    {
        this.maxBrowseDepth = maxBrowseDepth;
    }

    /**
     * Override the {@link #EVENT_NAME_DOCLIB_SELECT_FILE default} name for events
     * to navigate to a document library file
     */
    public void setEventNameDocLibSelectFile(String eventNameDocLibSelectFile)
    {
        this.eventNameDocLibSelectFile = eventNameDocLibSelectFile;
    }

    /**
     * Override the {@link #EVENT_NAME_DOCLIB_SELECT_FOLDER default} name for events
     * to navigate to a folder
     */
    public void setEventNameDocLibSelectFolder(String eventNameDocLibSelectFolder)
    {
        this.eventNameDocLibSelectFolder = eventNameDocLibSelectFolder;
    }

    /**
     * @see ShareCreateSiteScenario see class comments
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        SharePage sharePage = eventData.getSharePage();
        if (sharePage instanceof DocumentDetailsPage)
        {
            // We've hit a document.  End.
            return Collections.emptyList();
        }
        else if (sharePage instanceof DocumentLibraryPage)
        {
            if (eventData instanceof ShareSelectDocLibEntryEventData)
            {
                // We're visiting this scenario again.  Look at the depth and break out, if required.
                ShareSelectDocLibEntryEventData selectEventData = (ShareSelectDocLibEntryEventData) eventData;
                int depth = selectEventData.incrementDepth();
                if (depth >= maxBrowseDepth)
                {
                    // Do not go any deeper.  Break out.
                    return Collections.emptyList();
                }
                // We can continue down
            }
            DocumentLibraryPage documentLibraryPage = (DocumentLibraryPage) sharePage;
            // Are there any documents?
            List<FileDirectoryInfo> fileInfos = documentLibraryPage.getFiles();
            if (fileInfos.size() == 0)
            {
                // There is nothing to select
                return Collections.emptyList();
            }
            // Choose one of them
            int choice = (int) (Math.random() * (double)fileInfos.size());
            FileDirectoryInfo fileInfo = fileInfos.get(choice);
            String title = fileInfo.getName();
            eventData = new ShareSelectDocLibEntryEventData(eventData, title);
            Event nextEvent = null;
            // The type of event will depend on whether we have a file or folder
            if (fileInfo.isTypeFolder())
            {
                nextEvent = new Event(eventNameDocLibSelectFolder, nextEventTime, eventData);
            }
            else
            {
                nextEvent = new Event(eventNameDocLibSelectFile, nextEventTime, eventData);
            }
            // Done
            return Collections.singletonList(nextEvent);
        }
        else
        {
            // Follow the parent class's actions (to get to the document library)
            return super.getNextEvents(event, eventData, soakData, nextEventTime);
        }
    }
}
