/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.util.Collections;
import java.util.List;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.search.SearchResult;
import org.alfresco.po.share.search.SearchResultsPage;

/**
 * Selects a random entry in the search results page.
 * <p/>
 * The following page is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. Search results (with entries):</b> Chooses a random file <b>or</b> folder.</li>
 *   <li><b>   2.1. Site doclib:</b> END</li>
 *   <li><b>   2.2. Document details:</b> END</li>
 * </ul>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareSelectSearchEntryScenario extends AbstractShareScenario
{
    public static final String EVENT_NAME_SEARCH_SELECT_ENTRY = "share.search.selectEntry";

    private String eventNameSearchSelectEntry = EVENT_NAME_SEARCH_SELECT_ENTRY;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     */
    public ShareSelectSearchEntryScenario(ShareScenarioRegistry registry, double weight)
    {
        super(registry, weight);
    }

    /**
     * Override the {@link #EVENT_NAME_SEARCH_SELECT_ENTRY default} name for events
     * to select a search result entry
     */
    public void setEventNameSearchSelectEntry(String eventNameSearchSelectEntry)
    {
        this.eventNameSearchSelectEntry = eventNameSearchSelectEntry;
    }

    /**
     * @see ShareCreateSiteScenario see class comments
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        SharePage sharePage = eventData.getSharePage();
        if (sharePage instanceof SearchResultsPage)
        {
            SearchResultsPage searchResultsPage = (SearchResultsPage) sharePage;
            // Are there any documents?
            List<SearchResult> searchResults = searchResultsPage.getResults();
            if (searchResults.size() == 0)
            {
                // There is nothing to select
                return Collections.emptyList();
            }
            // Choose one of them
            int choice = (int) (Math.random() * (double)searchResults.size());
            SearchResult searchResult = searchResults.get(choice);
            String title = searchResult.getTitle();
            eventData.setFilename(title);
            Event nextEvent = new Event(eventNameSearchSelectEntry, nextEventTime, eventData);
            // Done
            return Collections.singletonList(nextEvent);
        }
        else
        {
            // Not a page of interest
            return Collections.emptyList();
        }
    }
}
